.nh
.TH indexer 1 "October 2021" manticoresearch-tools
.SH NAME
.PP
indexer - auxiliary tool to create plain indexes for Manticore Search

.SH BACKGROUND INFORMATION
.PP
Plain indexes are indexes that are created one-time by fetching data at creation from one or several sources. A plain index is immutable as documents cannot be added or deleted during it's lifespan. It is only possible to update values of numeric attributes (including MVA). Refreshing the data is only possible by recreating the whole index.

.PP
Plain indexes are available only in Plain mode and their definition is made of an index declaration and one or several source declarations. The data gathering and index creation is not made by the \fBsearchd\fR server, but by the auxiliary tool \fBindexer\fR\&.

.SH DESCRIPTION
.PP
indexer is a command line tool that can be called directly from the command line or from shell scripts.

.PP
It can accept a number of arguments when called, but there are also several settings of it's own in the Manticore configuration file.

.PP
In the typical scenario, indexer does the following:

.RS
.IP \(bu 2
fetches the data from the source
.IP \(bu 2
builds the plain index
.IP \(bu 2
writes the index files
.IP \(bu 2
(optional) informs search server about the new index which triggers index rotation

.RE

.PP
General syntax for \fBindexer\fR is as follows:

.EX
indexer [OPTIONS] [indexname1 [indexname2 [...]]]

.EE

.PP
An important thing to keep in mind when creating indexes with \fBindexer\fR is that the generated index files must be made with permissions that allow \fBsearchd\fR to read, write and even delete them. In case of Linux official packages \fBsearchd\fR runs under \fBmanticore\fR user. In this case \fBindexer\fR must also run under \fBmanticore\fR user:

.EX
sudo -u manticore indexer ...

.EE

.PP
If you are running \fBsearchd\fR (Manticore Search server) differently you might need to omit \fBsudo -u manticore\fR, just make sure that the user under which your \fBsearchd\fR instance is running has read/write permissions to your indexes that you generate using \fBindexer\fR\&.

.PP
Essentially you would list the different possible indexes (that you would later make available to search) in \fBmanticore.conf\fR, so when calling \fBindexer\fR, as a minimum you need to be telling it what index (or indexes) you want to index. If \fBmanticore.conf\fR contained details on 2 indexes, \fBmybigindex\fR and \fBmysmallindex\fR, you could do the following:

.EX
sudo -u manticore indexer mysmallindex mybigindex

.EE

.PP
Wildcarding on index names is also supported. The following wildcard tokens can be used:

.RS
.IP \(bu 2
\fB?\fR matches any single character
.IP \(bu 2
\fB*\fR matches any count of any characters
.IP \(bu 2
\fB%\fR matches none or any single character

.RE

.EX
sudo -u manticore indexer indexpart*main --rotate

.EE

.SH EXIT STATUS
.RS
.IP \(bu 2
0 - everything went ok
.IP \(bu 2
1 - there was a problem while indexing (and if –-rotate was specified, it was skipped) or an operation emitted a warning
.IP \(bu 2
2 - indexing went ok, but –-rotate attempt failed

.RE

.SH OPTIONS
.RS
.IP \(bu 2
\fB--config <file>\fR (\fB-c <file>\fR for short) tells \fBindexer\fR to use the given file as its configuration. Normally, it will look for \fBmanticore.conf\fR in the installation directory (e.g. \fB/etc/manticoresearch/manticore.conf\fR), followed by the current directory you are in when calling \fBindexer\fR from the shell. This is most of use in shared environments where the binary files are installed in a global folder, e.g. \fB/usr/bin/\fR, but you want to provide users with the ability to make their own custom Manticore set-ups, or if you want to run multiple instances on a single server. In cases like those you could allow them to create their own \fBmanticore.conf\fR files and pass them to \fBindexer\fR with this option. For example:

.RE

.EX
sudo -u manticore indexer --config /home/myuser/manticore.conf myindex

.EE

.RS
.IP \(bu 2
\fB--all\fR tells \fBindexer\fR to update every index listed in \fBmanticore.conf\fR instead of listing individual indexes. This would be useful in small configurations or cron-kind or maintenance jobs where the entire index set will get rebuilt each day or week or whatever period is best. Please note that since \fB--all\fR tries to update all found indexes in the configuration, it will issue a warning if encounters RealTime indexes and the exit code of the command will be \fB1\fR not \fB0\fR even if the plain indexes finished without issue. Example usage:

.RE

.EX
sudo -u manticore indexer --config /home/myuser/manticore.conf --all

.EE

.RS
.IP \(bu 2
\fB--rotate\fR is used for rotating indexes. Unless you have the situation where you can take the search function offline without troubling users you will almost certainly need to keep search running whilst indexing new documents. \fB--rotate\fR creates a second index, parallel to the first (in the same place, simply including \fB\&.new\fR in the filenames). Once complete, \fBindexer\fR notifies \fBsearchd\fR via sending the \fBSIGHUP\fR signal, and the \fBsearchd\fR will attempt to rename the indexes (renaming the existing ones to include \fB\&.old\fR and renaming the \fB\&.new\fR to replace them), and then will start serving from the newer files. Depending on the setting of seamless_rotate there may be a slight delay in being able to search the newer indexes. In case multiple indexes are rotated at once which are chained by killlist_target relations rotation will start with the indexes that are not targets and finish with the ones at the end of target chain. Example usage:

.RE

.EX
sudo -u manticore indexer --rotate --all

.EE

.RS
.IP \(bu 2
\fB--quiet\fR tells \fBindexer\fR not to output anything, unless there is an error. This is mostly used for cron-type or other scripted jobs where the output is irrelevant or unnecessary, except in the event of some kind of error. Example usage:

.RE

.EX
sudo -u manticore indexer --rotate --all --quiet

.EE

.RS
.IP \(bu 2
\fB--noprogress\fR does not display progress details as they occur. Instead, the final status details (such as documents indexed, speed of indexing and so on are only reported at completion of indexing. In instances where the script is not being run on a console (or 'tty'), this will be on by default. Example usage:

.RE

.EX
sudo -u manticore indexer --rotate --all --noprogress

.EE

.RS
.IP \(bu 2
\fB--buildstops <outputfile.text> <N>\fR reviews the index source, as if it were indexing the data, and produces a list of the terms that are being indexed. In other words, it produces a list of all the searchable terms that are becoming part of the index. Note, it does not update the index in question, it simply processes the data \fBas if\fP it were indexing, including running queries defined with sql_query_pre or sql_query_post. \fBoutputfile.txt\fR will contain the list of words, one per line, sorted by frequency with most frequent first, and \fBN\fR specifies the maximum number of words that will be listed. If it's sufficiently large to encompass every word in the index, only that many words will be returned. Such a dictionary list could be used for client application features around "Did you mean…" functionality, usually in conjunction with \fB--buildfreqs\fR, below. Example:

.RE

.EX
sudo -u manticore indexer myindex --buildstops word_freq.txt 1000

.EE

.PP
This would produce a document in the current directory, \fBword_freq.txt\fR with the 1,000 most common words in 'myindex', ordered by most common first. Note that the file will pertain to the last index indexed when specified with multiple indexes or \fB--all\fR (i.e. the last one listed in the configuration file)

.RS
.IP \(bu 2
\fB--buildfreqs\fR works with \fB--buildstops\fR (and is ignored if \fB--buildstops\fR is not specified). As \fB--buildstops\fR provides the list of words used within the index, \fB--buildfreqs\fR adds the quantity present in the index, which would be useful in establishing whether certain words should be considered stopwords if they are too prevalent. It will also help with developing "Did you mean…" features where you need to know how much more common a given word compared to another, similar one. Example:

.RE

.EX
sudo -u manticore indexer myindex --buildstops word_freq.txt 1000 --buildfreqs

.EE

.PP
This would produce the \fBword_freq.txt\fR as above, however after each word would be the number of times it occurred in the index in question.

.RS
.IP \(bu 2
\fB--merge <dst-index> <src-index>\fR is used for physically merging indexes together, for example if you have a main+delta scheme, where the main index rarely changes, but the delta index is rebuilt frequently, and \fB--merge\fR would be used to combine the two. The operation moves from right to left - the contents of \fBsrc-index\fR get examined and physically combined with the contents of \fBdst-index\fR and the result is left in \fBdst-index\fR\&. In pseudo-code, it might be expressed as: \fBdst-index += src-index\fR An example:

.RE

.EX
sudo -u manticore indexer --merge main delta --rotate

.EE

.PP
In the above example, where the main is the master, rarely modified index, and the delta is more frequently modified one, you might use the above to call \fBindexer\fR to combine the contents of the delta into the main index and rotate the indexes.

.RS
.IP \(bu 2
\fB--merge-dst-range <attr> <min> <max>\fR runs the filter range given upon merging. Specifically, as the merge is applied to the destination index (as part of \fB--merge\fR, and is ignored if \fB--merge\fR is not specified), \fBindexer\fR will also filter the documents ending up in the destination index, and only documents will pass through the filter given will end up in the final index. This could be used for example, in an index where there is a 'deleted' attribute, where 0 means 'not deleted'. Such an index could be merged with:

.RE

.EX
sudo -u manticore indexer --merge main delta --merge-dst-range deleted 0 0

.EE

.PP
Any documents marked as deleted (value 1) would be removed from the newly-merged destination index. It can be added several times to the command line, to add successive filters to the merge, all of which must be met in order for a document to become part of the final index.

.RS
.IP \(bu 2
\fB--merge-killlists\fR (and its shorter alias \fB--merge-klists\fR) changes the way kill lists are processed when merging indexes. By default, both kill lists get discarded after a merge. That supports the most typical main+delta merge scenario. With this option enabled, however, kill lists from both indexes get concatenated and stored into the destination index. Note that a source (delta) index kill list will be used to suppress rows from a destination (main) index at all times.
.IP \(bu 2
\fB--keep-attrs\fR allows to reuse existing attributes on reindexing. Whenever the index is rebuilt, each new document id is checked for presence in the "old" index, and if it already exists, its attributes are transferred to the "new" index; if not found, attributes from the new index are used. If the user has updated attributes in the index, but not in the actual source used for the index, all updates will be lost when reindexing; using –keep-attrs enables saving the updated attribute values from the previous index. It is possible to specify a path for index files to used instead of reference path from config:

.RE

.EX
sudo -u manticore indexer myindex --keep-attrs=/path/to/index/files

.EE

.RS
.IP \(bu 2
\fB--keep-attrs-names=<attributes list>\fR allows to specify attributes to reuse from existing index on reindexing. By default all attributes from existed index reused at new "index":

.RE

.EX
sudo -u manticore indexer myindex --keep-attrs=/path/to/index/files --keep-attrs-names=update,state

.EE

.RS
.IP \(bu 2
\fB--dump-rows <FILE>\fR dumps rows fetched by SQL source(s) into the specified file, in a MySQL compatible syntax. Resulting dumps are the exact representation of data as received by \fBindexer\fR and help to repeat indexing-time issues. The command performs fetching from the source and creates both index files and the dump file.
.IP \(bu 2
\fB--print-rt <rt_index> <index>\fR outputs fetched data from source as INSERTs for a real-time index. The first lines of the dump will contain the real-time fields and attributes (as a reflection of the plain index fields and attributes). The command performs fetching from the source and creates both index files and the dump output. The command can be used as \fBsudo -u manticore indexer -c manticore.conf --print-rt indexrt indexplain > dump.sql\fR\&. Only sql-based sources are supported. MVAs are not supported.
.IP \(bu 2
\fB--sighup-each\fR is useful when you are rebuilding many big indexes, and want each one rotated into \fBsearchd\fR as soon as possible. With \fB--sighup-each\fR, \fBindexer\fR will send the SIGHUP signal to searchd after successfully completing work on each index. (The default behavior is to send a single SIGHUP after all the indexes are built).
.IP \(bu 2
\fB--nohup\fR is useful when you want to check your index with indextool before actually rotating it. indexer won't send the SIGHUP if this option is on. Index files are renamed to .tmp. Use indextool to rename index files to .new and rotate it. Example usage:

.RE

.EX
sudo -u manticore indexer --rotate --nohup myindex
sudo -u manticore indextool --rotate --check myindex

.EE

.RS
.IP \(bu 2
\fB--print-queries\fR prints out SQL queries that \fBindexer\fR sends to the database, along with SQL connection and disconnection events. That is useful to diagnose and fix problems with SQL sources.
.IP \(bu 2
\fB--help\fR (\fB-h\fR for short) lists all of the parameters that can be called in \fBindexer\fR\&.
.IP \(bu 2
\fB-v\fR shows \fBindexer\fR version.

.RE
